﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Infragistics.Windows.Ribbon;
using Infragistics.Windows.Controls;

namespace Infragistics.Windows.Extensions
{
	/// <summary>
	/// Static helper class for working with <see cref="XamRibbon"/>
	/// </summary>
	public static class RibbonHelper
	{
		#region Constructor
		static RibbonHelper()
		{
			EventManager.RegisterClassHandler(typeof(FrameworkElement), ToolTipService.ToolTipOpeningEvent, new ToolTipEventHandler(OnToolTipOpening));
		}
		#endregion //Constructor

		#region Properties

		#region SuppressSimpleMenuToolTips

		/// <summary>
		/// SuppressSimpleMenuToolTips Attached Dependency Property
		/// </summary>
		public static readonly DependencyProperty SuppressSimpleMenuToolTipsProperty =
			DependencyProperty.RegisterAttached("SuppressSimpleMenuToolTips", typeof(bool), typeof(RibbonHelper),
				new FrameworkPropertyMetadata(false));

		/// <summary>
		/// Gets a value indicating if tooltips where the value matches the tool caption should be suppressed
		/// </summary>
		public static bool GetSuppressSimpleMenuToolTips(DependencyObject d)
		{
			return (bool)d.GetValue(SuppressSimpleMenuToolTipsProperty);
		}

		/// <summary>
		/// Sets a value that indicates if a tooltip should be suppressed for a menu item when the tool caption matches the tooltip.
		/// </summary>
		public static void SetSuppressSimpleMenuToolTips(DependencyObject d, bool value)
		{
			d.SetValue(SuppressSimpleMenuToolTipsProperty, value);
		}

		#endregion //SuppressSimpleMenuToolTips

		#endregion //Properties

		#region Methods

		#region OnToolTipOpening
		private static void OnToolTipOpening(object sender, ToolTipEventArgs e)
		{
			XamRibbon ribbon = XamRibbon.GetRibbon(sender as DependencyObject);

			if (ribbon != null)
			{
				ProxyEventArgs<ToolTipEventArgs> args = new ProxyEventArgs<ToolTipEventArgs>(e);

				args.RoutedEvent = RibbonToolTipOpeningEvent;
				args.Source = ribbon;
				ribbon.RaiseEvent(args);

				if (!e.Handled && GetSuppressSimpleMenuToolTips(ribbon) && ShouldSuppressToolTip(e))
					e.Handled = true;
			}
		}
		#endregion //OnToolTipOpening

		#region ShouldSuppressToolTip
		private static bool ShouldSuppressToolTip(ToolTipEventArgs e)
		{
			DependencyObject originalSource = e.OriginalSource as DependencyObject;

			if (originalSource != null)
			{
				// for the tooltipopening event, setting the Handled to true means that 
				// you don't want the wpf framework to show the tooltip so we can use 
				// this to suppress the tooltip
				switch (XamRibbon.GetLocation(originalSource))
				{
					case ToolLocation.ApplicationMenu:
					case ToolLocation.ApplicationMenuRecentItems:
					case ToolLocation.ApplicationMenuSubMenu:
					case ToolLocation.Menu:
						ToolMenuItem tmi = originalSource as ToolMenuItem;

						if (tmi != null && ShouldSuppressToolTip(tmi.Header, tmi.ToolTip))
							return true;
						break;

					case ToolLocation.ApplicationMenuFooterToolbar:
						FrameworkElement fe = originalSource as FrameworkElement;
						if (fe != null && ShouldSuppressToolTip(RibbonToolHelper.GetCaption(fe), fe.ToolTip))
							return true;
						break;
				}
			}

			return false;
		}

		private static bool ShouldSuppressToolTip(object caption, object tooltip)
		{
			// if a complex tip is specified then let it show
			if (tooltip is XamScreenTip)
				return false;

			ToolTip tt = tooltip as ToolTip;

			if (null != tt)
				tooltip = tt.Content;

			if (object.Equals(caption, tooltip))
				return true;

			string captionString = caption as String;

			if (null != captionString)
			{
				captionString = Utilities.StripMnemonics(captionString, false);

				if (object.Equals(captionString, tooltip))
					return true;
			}

			return false;
		}
		#endregion //ShouldSuppressToolTip

		#endregion //Methods

		#region Events

		#region RibbonToolTipOpening

		/// <summary>
		/// RibbonToolTipOpening Attached Routed Event
		/// </summary>
		public static readonly RoutedEvent RibbonToolTipOpeningEvent = EventManager.RegisterRoutedEvent("RibbonToolTipOpening",
			RoutingStrategy.Direct, typeof(EventHandler<ProxyEventArgs<ToolTipEventArgs>>), typeof(RibbonHelper));

		/// <summary>
		/// Adds a handler for the RibbonToolTipOpening attached event
		/// </summary>
		/// <param name="element">UIElement or ContentElement that listens to the event</param>
		/// <param name="handler">Event handler to be added</param>
		public static void AddRibbonToolTipOpeningHandler(DependencyObject element, EventHandler<ProxyEventArgs<ToolTipEventArgs>> handler)
		{
			UIElementHelper.AddHandler(element, RibbonToolTipOpeningEvent, handler);
		}

		/// <summary>
		/// Removes a handler for the RibbonToolTipOpening attached event
		/// </summary>
		/// <param name="element">UIElement or ContentElement that listens to the event</param>
		/// <param name="handler">Event handler to be removed</param>
		public static void RemoveRibbonToolTipOpeningHandler(DependencyObject element, EventHandler<ProxyEventArgs<ToolTipEventArgs>> handler)
		{
			UIElementHelper.RemoveHandler(element, RibbonToolTipOpeningEvent, handler);
		}

		#endregion //RibbonToolTipOpening

		#endregion //Events
	}
}
